﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using ThinkGeo.MapSuite.SilverlightCore;

namespace ThinkGeo.MapSuite.SilverlightEdition
{
    public class CustomMarkerStyle : PointMarkerStyle
    {
        public string ImageColumn { get; set; }
        public Map CurrentMap { get; set; }

        public override Collection<Marker> GetMarkers(IEnumerable<Feature> features)
        {
            Collection<Marker> returnMarkers = new Collection<Marker>();
            foreach(var feature in features)
            {
                returnMarkers.Add(CreateMarker(feature.GetBoundingBox().GetCenterPoint(), feature.Id, feature.ColumnValues));
            }

            return returnMarkers;
        }

        private Marker CreateMarker(PointShape point, string markerId, Dictionary<string, string> columnValues)
        {
            Marker marker = new Marker(point);
            marker.Name = markerId;
            marker.Popup.Position = new Point(point.X, point.Y);
            marker.Popup = Popup;
            marker.Popup.SetValue(Canvas.ZIndexProperty, 801);
       
            marker.ImageSource = GetImageSource(columnValues[ImageColumn]);
            if (!double.IsPositiveInfinity(ImageWidth))
            {
                marker.ImageWidth = ImageWidth;
            }
            if (!double.IsPositiveInfinity(ImageHeight))
            {
                marker.ImageHeight = ImageHeight;
            }
            marker.ImageOffsetX = ImageOffsetX;
            marker.ImageOffsetY = ImageOffsetY;
            marker.ImageRotationAngle = ImageRotateAngle;
            marker.TextRotationAngle = TextRotateAngle;
            marker.TextWrapping = this.TextWrapping;
            marker.TextOffsetX = TextOffsetX;
            marker.TextOffsetY = TextOffsetY;
            marker.TextWidth = TextWidth;

            var scaleTransform = new ScaleTransform() { ScaleX = CurrentMap.CurrentZoomLevelId, ScaleY = CurrentMap.CurrentZoomLevelId};
            marker.RenderTransform = scaleTransform;

            if (!string.IsNullOrEmpty(marker.Text))
            {
                foreach (string key in columnValues.Keys)
                {
                    marker.Text = marker.Text.Replace("[#" + key + "#]", columnValues[key].ToString());
                }
            }

            return marker;
        }

        private ImageSource GetImageSource(string value)
        {
            ImageSource imageSource = null;
            switch (value)
            {
                case "RED":
                    imageSource = new BitmapImage(new Uri("resource/marker_red_shadow.png", UriKind.RelativeOrAbsolute));
                    break;

                case "GREEN":
                default:
                    imageSource = new BitmapImage(new Uri("resource/marker_green_shadow.png", UriKind.RelativeOrAbsolute));
                    break;
            }

            return imageSource;
        }
    }
}
